@@ -3,6 +3,8 @@
 #' Use a cache on the local filesystem that will persist between R sessions.
 #'
 #' @param path Directory in which to store cached items.
+#' @param compress Argument passed to \code{saveRDS}. One of FALSE, "gzip",
+#' "bzip2" or "xz". Default: FALSE.
 #'
 #' @examples
 #'
@@ -23,19 +25,23 @@
 #'
 #' @export
 #' @inheritParams cache_memory
-cache_filesystem <- function(path, algo = "xxhash64") {
+cache_filesystem <- function(path, algo = "xxhash64", compress = FALSE) {
+  if (!(requireNamespace("digest"))) { stop("Package `digest` must be installed for `cache_filesystem()`.") } # nocov
 
   if (!dir.exists(path)) {
     dir.create(path, showWarnings = FALSE)
   }
 
+  # convert to absolute path so it will work with user working directory changes
+  path <- normalizePath(path)
+
   cache_reset <- function() {
     cache_files <- list.files(path, full.names = TRUE)
     file.remove(cache_files)
   }
 
   cache_set <- function(key, value) {
-    saveRDS(value, file = file.path(path, key))
+    saveRDS(value, file = file.path(path, key), compress = compress)
   }
 
   cache_get <- function(key) {
@@ -46,12 +52,17 @@ cache_filesystem <- function(path, algo = "xxhash64") {
     file.exists(file.path(path, key))
   }
 
+  cache_drop_key <- function(key) {
+    file.remove(file.path(path, key))
+  }
+
   list(
     digest = function(...) digest::digest(..., algo = algo),
     reset = cache_reset,
     set = cache_set,
     get = cache_get,
     has_key = cache_has_key,
+    drop_key = cache_drop_key,
     keys = function() list.files(path)
   )
 }
